# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .cohort_response import CohortResponse
    from .lang2fhir_and_create_multi_response import Lang2FhirAndCreateMultiResponse
    from .lang2fhir_and_create_multi_response_resource_info_item import Lang2FhirAndCreateMultiResponseResourceInfoItem
    from .lang2fhir_and_create_multi_response_response_bundle import Lang2FhirAndCreateMultiResponseResponseBundle
    from .lang2fhir_and_create_request_resource import Lang2FhirAndCreateRequestResource
    from .lang2fhir_and_create_response import Lang2FhirAndCreateResponse
    from .lang2fhir_and_search_response import Lang2FhirAndSearchResponse
    from .mcp_server_response import McpServerResponse
    from .mcp_server_response_data import McpServerResponseData
    from .mcp_server_tool_call_response import McpServerToolCallResponse
    from .mcp_server_tool_response import McpServerToolResponse
    from .mcp_server_tool_response_data import McpServerToolResponseData
    from .search_concept import SearchConcept
_dynamic_imports: typing.Dict[str, str] = {
    "CohortResponse": ".cohort_response",
    "Lang2FhirAndCreateMultiResponse": ".lang2fhir_and_create_multi_response",
    "Lang2FhirAndCreateMultiResponseResourceInfoItem": ".lang2fhir_and_create_multi_response_resource_info_item",
    "Lang2FhirAndCreateMultiResponseResponseBundle": ".lang2fhir_and_create_multi_response_response_bundle",
    "Lang2FhirAndCreateRequestResource": ".lang2fhir_and_create_request_resource",
    "Lang2FhirAndCreateResponse": ".lang2fhir_and_create_response",
    "Lang2FhirAndSearchResponse": ".lang2fhir_and_search_response",
    "McpServerResponse": ".mcp_server_response",
    "McpServerResponseData": ".mcp_server_response_data",
    "McpServerToolCallResponse": ".mcp_server_tool_call_response",
    "McpServerToolResponse": ".mcp_server_tool_response",
    "McpServerToolResponseData": ".mcp_server_tool_response_data",
    "SearchConcept": ".search_concept",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "CohortResponse",
    "Lang2FhirAndCreateMultiResponse",
    "Lang2FhirAndCreateMultiResponseResourceInfoItem",
    "Lang2FhirAndCreateMultiResponseResponseBundle",
    "Lang2FhirAndCreateRequestResource",
    "Lang2FhirAndCreateResponse",
    "Lang2FhirAndSearchResponse",
    "McpServerResponse",
    "McpServerResponseData",
    "McpServerToolCallResponse",
    "McpServerToolResponse",
    "McpServerToolResponseData",
    "SearchConcept",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .error_response import ErrorResponse
    from .fhir_bundle import FhirBundle
    from .fhir_bundle_entry_item import FhirBundleEntryItem
    from .fhir_bundle_entry_item_request import FhirBundleEntryItemRequest
    from .fhir_bundle_entry_item_request_method import FhirBundleEntryItemRequestMethod
    from .fhir_bundle_entry_item_response import FhirBundleEntryItemResponse
    from .fhir_patch_request_body_item import FhirPatchRequestBodyItem
    from .fhir_patch_request_body_item_op import FhirPatchRequestBodyItemOp
    from .fhir_resource import FhirResource
    from .fhir_resource_meta import FhirResourceMeta
    from .fhir_search_response import FhirSearchResponse
_dynamic_imports: typing.Dict[str, str] = {
    "ErrorResponse": ".error_response",
    "FhirBundle": ".fhir_bundle",
    "FhirBundleEntryItem": ".fhir_bundle_entry_item",
    "FhirBundleEntryItemRequest": ".fhir_bundle_entry_item_request",
    "FhirBundleEntryItemRequestMethod": ".fhir_bundle_entry_item_request_method",
    "FhirBundleEntryItemResponse": ".fhir_bundle_entry_item_response",
    "FhirPatchRequestBodyItem": ".fhir_patch_request_body_item",
    "FhirPatchRequestBodyItemOp": ".fhir_patch_request_body_item_op",
    "FhirResource": ".fhir_resource",
    "FhirResourceMeta": ".fhir_resource_meta",
    "FhirSearchResponse": ".fhir_search_response",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "ErrorResponse",
    "FhirBundle",
    "FhirBundleEntryItem",
    "FhirBundleEntryItemRequest",
    "FhirBundleEntryItemRequestMethod",
    "FhirBundleEntryItemResponse",
    "FhirPatchRequestBodyItem",
    "FhirPatchRequestBodyItemOp",
    "FhirResource",
    "FhirResourceMeta",
    "FhirSearchResponse",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        CohortResponse,
        Lang2FhirAndCreateMultiResponse,
        Lang2FhirAndCreateMultiResponseResourceInfoItem,
        Lang2FhirAndCreateMultiResponseResponseBundle,
        Lang2FhirAndCreateRequestResource,
        Lang2FhirAndCreateResponse,
        Lang2FhirAndSearchResponse,
        McpServerResponse,
        McpServerResponseData,
        McpServerToolCallResponse,
        McpServerToolResponse,
        McpServerToolResponseData,
        SearchConcept,
    )
    from .errors import BadRequestError, FailedDependencyError, ForbiddenError, InternalServerError, UnauthorizedError
    from . import mcp_server
_dynamic_imports: typing.Dict[str, str] = {
    "BadRequestError": ".errors",
    "CohortResponse": ".types",
    "FailedDependencyError": ".errors",
    "ForbiddenError": ".errors",
    "InternalServerError": ".errors",
    "Lang2FhirAndCreateMultiResponse": ".types",
    "Lang2FhirAndCreateMultiResponseResourceInfoItem": ".types",
    "Lang2FhirAndCreateMultiResponseResponseBundle": ".types",
    "Lang2FhirAndCreateRequestResource": ".types",
    "Lang2FhirAndCreateResponse": ".types",
    "Lang2FhirAndSearchResponse": ".types",
    "McpServerResponse": ".types",
    "McpServerResponseData": ".types",
    "McpServerToolCallResponse": ".types",
    "McpServerToolResponse": ".types",
    "McpServerToolResponseData": ".types",
    "SearchConcept": ".types",
    "UnauthorizedError": ".errors",
    "mcp_server": ".mcp_server",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "BadRequestError",
    "CohortResponse",
    "FailedDependencyError",
    "ForbiddenError",
    "InternalServerError",
    "Lang2FhirAndCreateMultiResponse",
    "Lang2FhirAndCreateMultiResponseResourceInfoItem",
    "Lang2FhirAndCreateMultiResponseResponseBundle",
    "Lang2FhirAndCreateRequestResource",
    "Lang2FhirAndCreateResponse",
    "Lang2FhirAndSearchResponse",
    "McpServerResponse",
    "McpServerResponseData",
    "McpServerToolCallResponse",
    "McpServerToolResponse",
    "McpServerToolResponseData",
    "SearchConcept",
    "UnauthorizedError",
    "mcp_server",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        ErrorResponse,
        FhirBundle,
        FhirBundleEntryItem,
        FhirBundleEntryItemRequest,
        FhirBundleEntryItemRequestMethod,
        FhirBundleEntryItemResponse,
        FhirPatchRequestBodyItem,
        FhirPatchRequestBodyItemOp,
        FhirResource,
        FhirResourceMeta,
        FhirSearchResponse,
    )
    from .errors import (
        BadGatewayError,
        BadRequestError,
        InternalServerError,
        NotFoundError,
        ServiceUnavailableError,
        TooManyRequestsError,
        UnauthorizedError,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "BadGatewayError": ".errors",
    "BadRequestError": ".errors",
    "ErrorResponse": ".types",
    "FhirBundle": ".types",
    "FhirBundleEntryItem": ".types",
    "FhirBundleEntryItemRequest": ".types",
    "FhirBundleEntryItemRequestMethod": ".types",
    "FhirBundleEntryItemResponse": ".types",
    "FhirPatchRequestBodyItem": ".types",
    "FhirPatchRequestBodyItemOp": ".types",
    "FhirResource": ".types",
    "FhirResourceMeta": ".types",
    "FhirSearchResponse": ".types",
    "InternalServerError": ".errors",
    "NotFoundError": ".errors",
    "ServiceUnavailableError": ".errors",
    "TooManyRequestsError": ".errors",
    "UnauthorizedError": ".errors",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "BadGatewayError",
    "BadRequestError",
    "ErrorResponse",
    "FhirBundle",
    "FhirBundleEntryItem",
    "FhirBundleEntryItemRequest",
    "FhirBundleEntryItemRequestMethod",
    "FhirBundleEntryItemResponse",
    "FhirPatchRequestBodyItem",
    "FhirPatchRequestBodyItemOp",
    "FhirResource",
    "FhirResourceMeta",
    "FhirSearchResponse",
    "InternalServerError",
    "NotFoundError",
    "ServiceUnavailableError",
    "TooManyRequestsError",
    "UnauthorizedError",
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from .raw_client import AsyncRawConstrueClient, RawConstrueClient
from .types.code_response import CodeResponse
from .types.construe_upload_code_system_response import ConstrueUploadCodeSystemResponse
from .types.delete_code_system_response import DeleteCodeSystemResponse
from .types.export_code_system_response import ExportCodeSystemResponse
from .types.extract_codes_result import ExtractCodesResult
from .types.extract_request_config import ExtractRequestConfig
from .types.extract_request_system import ExtractRequestSystem
from .types.feedback_response import FeedbackResponse
from .types.get_code_response import GetCodeResponse
from .types.get_code_system_detail_response import GetCodeSystemDetailResponse
from .types.list_code_systems_response import ListCodeSystemsResponse
from .types.list_codes_response import ListCodesResponse
from .types.semantic_search_response import SemanticSearchResponse
from .types.text_search_response import TextSearchResponse
from .types.upload_request_format import UploadRequestFormat

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ConstrueClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawConstrueClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawConstrueClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawConstrueClient
        """
        return self._raw_client

    def upload_code_system(
        self,
        *,
        name: str,
        version: str,
        format: UploadRequestFormat,
        revision: typing.Optional[float] = OMIT,
        file: typing.Optional[str] = OMIT,
        code_col: typing.Optional[str] = OMIT,
        desc_col: typing.Optional[str] = OMIT,
        defn_col: typing.Optional[str] = OMIT,
        codes: typing.Optional[typing.Sequence[CodeResponse]] = OMIT,
        replace: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConstrueUploadCodeSystemResponse:
        """
        Upload a custom medical code system with codes and descriptions for use in code extraction. Requires a paid plan.
        Returns 202 immediately; embedding generation runs asynchronously. Poll
        GET /construe/codes/systems/{codesystem}?version={version} to check when status
        transitions from "processing" to "ready" or "failed".

        Parameters
        ----------
        name : str
            Name of the code system. Names are case-insensitive and stored uppercase.
            Builtin system names (e.g. ICD-10-CM, SNOMED_CT_US_LITE, LOINC, CPT, etc.) are
            reserved and cannot be used for custom uploads; attempts return HTTP 403 Forbidden.

        version : str
            Version of the code system

        format : UploadRequestFormat
            Upload format

        revision : typing.Optional[float]
            Optional revision number

        file : typing.Optional[str]
            The file contents as a base64-encoded string.
            For CSV format, this is the CSV file contents.
            For JSON format, this is a base64-encoded JSON array; prefer using 'codes' instead.

        code_col : typing.Optional[str]
            Column name containing codes (required for CSV format)

        desc_col : typing.Optional[str]
            Column name containing descriptions (required for CSV format)

        defn_col : typing.Optional[str]
            Optional column name containing long definitions (for CSV format)

        codes : typing.Optional[typing.Sequence[CodeResponse]]
            The codes to upload as a JSON array (JSON format only).
            This is the preferred way to upload JSON codes, as it avoids unnecessary base64 encoding.
            If both 'codes' and 'file' are provided, 'codes' takes precedence.

        replace : typing.Optional[bool]
            If true, replaces an existing code system with the same name and version.
            Builtin systems cannot be replaced; attempts to do so return HTTP 403 Forbidden.
            When false (default), uploading a duplicate returns 409 Conflict.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConstrueUploadCodeSystemResponse
            Upload accepted for asynchronous processing

        Examples
        --------
        from phenoml import PhenomlClient

        client = PhenomlClient()
        client.construe.upload_code_system(
            name="CUSTOM_CODES",
            version="1.0",
            format="csv",
        )
        """
        _response = self._raw_client.upload_code_system(
            name=name,
            version=version,
            format=format,
            revision=revision,
            file=file,
            code_col=code_col,
            desc_col=desc_col,
            defn_col=defn_col,
            codes=codes,
            replace=replace,
            request_options=request_options,
        )
        return _response.data

    def extract_codes(
        self,
        *,
        text: str,
        system: typing.Optional[ExtractRequestSystem] = OMIT,
        config: typing.Optional[ExtractRequestConfig] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExtractCodesResult:
        """
        Converts natural language text into structured medical codes.

        Usage of CPT is subject to AMA requirements: see PhenoML Terms of Service.

        Parameters
        ----------
        text : str
            Natural language text to extract codes from

        system : typing.Optional[ExtractRequestSystem]

        config : typing.Optional[ExtractRequestConfig]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExtractCodesResult
            Successfully extracted codes

        Examples
        --------
        from phenoml import PhenomlClient

        client = PhenomlClient()
        client.construe.extract_codes(
            text="Patient is a 14-year-old female, previously healthy, who is here for evaluation of abnormal renal ultrasound with atrophic right kidney",
        )
        """
        _response = self._raw_client.extract_codes(
            text=text, system=system, config=config, request_options=request_options
        )
        return _response.data

    def list_available_code_systems(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ListCodeSystemsResponse:
        """
        Returns the terminology server's catalog of available code systems, including both built-in standard terminologies and custom uploaded systems.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListCodeSystemsResponse
            List of available code systems

        Examples
        --------
        from phenoml import PhenomlClient

        client = PhenomlClient()
        client.construe.list_available_code_systems()
        """
        _response = self._raw_client.list_available_code_systems(request_options=request_options)
        return _response.data

    def get_code_system_detail(
        self,
        codesystem: str,
        *,
        version: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetCodeSystemDetailResponse:
        """
        Returns full metadata for a single code system, including timestamps and builtin status.

        Parameters
        ----------
        codesystem : str
            Code system name (e.g., "ICD-10-CM", "SNOMED_CT_US_LITE")

        version : typing.Optional[str]
            Specific version of the code system. Required if multiple versions exist.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetCodeSystemDetailResponse
            Code system detail

        Examples
        --------
        from phenoml import PhenomlClient

        client = PhenomlClient()
        client.construe.get_code_system_detail(
            codesystem="ICD-10-CM",
            version="2025",
        )
        """
        _response = self._raw_client.get_code_system_detail(
            codesystem, version=version, request_options=request_options
        )
        return _response.data

    def delete_custom_code_system(
        self,
        codesystem: str,
        *,
        version: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DeleteCodeSystemResponse:
        """
        Deletes a custom (non-builtin) code system and all its codes. Builtin systems cannot be deleted.
        Only available on dedicated instances. Large systems may take up to a minute to delete.

        Parameters
        ----------
        codesystem : str
            Code system name

        version : typing.Optional[str]
            Specific version of the code system. Required if multiple versions exist.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteCodeSystemResponse
            Code system deleted successfully

        Examples
        --------
        from phenoml import PhenomlClient

        client = PhenomlClient()
        client.construe.delete_custom_code_system(
            codesystem="CUSTOM_CODES",
            version="version",
        )
        """
        _response = self._raw_client.delete_custom_code_system(
            codesystem, version=version, request_options=request_options
        )
        return _response.data

    def export_custom_code_system(
        self,
        codesystem: str,
        *,
        version: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExportCodeSystemResponse:
        """
        Exports a custom (non-builtin) code system as a JSON file compatible with the upload format.
        The exported file can be re-uploaded directly via POST /construe/upload with format "json".
        Only available on dedicated instances. Builtin systems cannot be exported.

        Parameters
        ----------
        codesystem : str
            Code system name

        version : typing.Optional[str]
            Specific version of the code system. Required if multiple versions exist.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExportCodeSystemResponse
            Exported code system as JSON file

        Examples
        --------
        from phenoml import PhenomlClient

        client = PhenomlClient()
        client.construe.export_custom_code_system(
            codesystem="CUSTOM_CODES",
            version="version",
        )
        """
        _response = self._raw_client.export_custom_code_system(
            codesystem, version=version, request_options=request_options
        )
        return _response.data

    def list_codes_in_a_code_system(
        self,
        codesystem: str,
        *,
        version: typing.Optional[str] = None,
        cursor: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListCodesResponse:
        """
        Returns a paginated list of all codes in the specified code system from the terminology server.

        Usage of CPT is subject to AMA requirements: see PhenoML Terms of Service.

        Parameters
        ----------
        codesystem : str
            Code system name (e.g., "ICD-10-CM", "SNOMED_CT_US_LITE")

        version : typing.Optional[str]
            Specific version of the code system. Required if multiple versions exist.

        cursor : typing.Optional[str]
            Pagination cursor from previous response

        limit : typing.Optional[int]
            Maximum number of codes to return (default 20)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListCodesResponse
            Paginated list of codes

        Examples
        --------
        from phenoml import PhenomlClient

        client = PhenomlClient()
        client.construe.list_codes_in_a_code_system(
            codesystem="ICD-10-CM",
            version="2025",
            cursor="cursor",
            limit=1,
        )
        """
        _response = self._raw_client.list_codes_in_a_code_system(
            codesystem, version=version, cursor=cursor, limit=limit, request_options=request_options
        )
        return _response.data

    def get_a_specific_code(
        self,
        codesystem: str,
        code_id: str,
        *,
        version: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetCodeResponse:
        """
        Looks up a specific code in the terminology server and returns its details.

        Usage of CPT is subject to AMA requirements: see PhenoML Terms of Service.

        Parameters
        ----------
        codesystem : str
            Code system name

        code_id : str
            The code identifier

        version : typing.Optional[str]
            Specific version of the code system

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetCodeResponse
            Code details

        Examples
        --------
        from phenoml import PhenomlClient

        client = PhenomlClient()
        client.construe.get_a_specific_code(
            codesystem="ICD-10-CM",
            code_id="E11.65",
            version="version",
        )
        """
        _response = self._raw_client.get_a_specific_code(
            codesystem, code_id, version=version, request_options=request_options
        )
        return _response.data

    def semantic_search_embedding_based(
        self,
        codesystem: str,
        *,
        text: str,
        version: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SemanticSearchResponse:
        """
        Performs semantic similarity search using vector embeddings.

        **Availability**: This endpoint works for both **built-in and custom** code systems.

        **When to use**: Best for natural language queries where you want to find conceptually
        related codes, even when different terminology is used. The search understands meaning,
        not just keywords.

        **Examples**:
        - Query "trouble breathing at night" finds codes like "Sleep apnea", "Orthopnea",
          "Nocturnal dyspnea" — semantically related but no exact keyword matches
        - Query "heart problems" finds "Myocardial infarction", "Cardiac arrest", "Arrhythmia"

        **Trade-offs**: Slower than text search (requires embedding generation), but finds
        conceptually similar results that keyword search would miss.

        See also: `/search/text` for faster keyword-based lookup with typo tolerance.

        Usage of CPT is subject to AMA requirements: see PhenoML Terms of Service.

        Parameters
        ----------
        codesystem : str
            Code system name

        text : str
            Natural language text to find semantically similar codes for

        version : typing.Optional[str]
            Specific version of the code system

        limit : typing.Optional[int]
            Maximum number of results (default 10, max 50)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SemanticSearchResponse
            Semantic search results ordered by similarity

        Examples
        --------
        from phenoml import PhenomlClient

        client = PhenomlClient()
        client.construe.semantic_search_embedding_based(
            codesystem="ICD-10-CM",
            text="patient has trouble breathing at night and wakes up gasping",
            version="version",
            limit=1,
        )
        """
        _response = self._raw_client.semantic_search_embedding_based(
            codesystem, text=text, version=version, limit=limit, request_options=request_options
        )
        return _response.data

    def submit_feedback_on_extraction_results(
        self,
        *,
        text: str,
        received_result: ExtractCodesResult,
        expected_result: ExtractCodesResult,
        detail: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FeedbackResponse:
        """
        Submits user feedback on results from the Construe extraction endpoint.
        Feedback includes the full extraction result received and the result the user expected.

        Parameters
        ----------
        text : str
            The natural language text that was used for code extraction

        received_result : ExtractCodesResult

        expected_result : ExtractCodesResult

        detail : typing.Optional[str]
            Optional details explaining the feedback

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FeedbackResponse
            Feedback saved successfully

        Examples
        --------
        from phenoml import PhenomlClient
        from phenoml.construe import (
            ExtractCodesResult,
            ExtractedCodeResult,
            ExtractRequestSystem,
        )

        client = PhenomlClient()
        client.construe.submit_feedback_on_extraction_results(
            text="Patient has type 2 diabetes with hyperglycemia",
            received_result=ExtractCodesResult(
                system=ExtractRequestSystem(),
                codes=[
                    ExtractedCodeResult(
                        code="195967001",
                        description="Asthma",
                        valid=True,
                    )
                ],
            ),
            expected_result=ExtractCodesResult(
                system=ExtractRequestSystem(),
                codes=[
                    ExtractedCodeResult(
                        code="195967001",
                        description="Asthma",
                        valid=True,
                    )
                ],
            ),
        )
        """
        _response = self._raw_client.submit_feedback_on_extraction_results(
            text=text,
            received_result=received_result,
            expected_result=expected_result,
            detail=detail,
            request_options=request_options,
        )
        return _response.data

    def terminology_server_text_search(
        self,
        codesystem: str,
        *,
        q: str,
        version: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TextSearchResponse:
        """
        Performs fast full-text search over code IDs and descriptions.

        **Availability**: This endpoint is only available for **built-in code systems**.
        Custom code systems uploaded via `/construe/upload` are not indexed for full-text search
        and will return empty results. Use `/search/semantic` to search custom code systems.

        **When to use**: Best for autocomplete UIs, code lookup, or when users know part of
        the code ID or specific keywords. Fast response times suitable for typeahead interfaces.

        **Features**:
        - Substring matching on code IDs (e.g., "11.65" finds "E11.65")
        - Typo tolerance on descriptions (not on code IDs)
        - Fast response times (~10-50ms)

        **Examples**:
        - Query "E11" finds all codes starting with E11 (diabetes codes)
        - Query "diabtes" (typo) still finds "diabetes" codes

        **Trade-offs**: Faster than semantic search, but only matches keywords/substrings.
        Won't find conceptually related codes with different terminology.

        See also: `/search/semantic` for finding conceptually similar codes.

        Usage of CPT is subject to AMA requirements: see PhenoML Terms of Service.

        Parameters
        ----------
        codesystem : str
            Code system name

        q : str
            Search query (searches code IDs and descriptions)

        version : typing.Optional[str]
            Specific version of the code system

        limit : typing.Optional[int]
            Maximum number of results (default 20, max 100)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TextSearchResponse
            Text search results

        Examples
        --------
        from phenoml import PhenomlClient

        client = PhenomlClient()
        client.construe.terminology_server_text_search(
            codesystem="ICD-10-CM",
            q="E11.65",
            version="version",
            limit=1,
        )
        """
        _response = self._raw_client.terminology_server_text_search(
            codesystem, q=q, version=version, limit=limit, request_options=request_options
        )
        return _response.data


class AsyncConstrueClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawConstrueClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawConstrueClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawConstrueClient
        """
        return self._raw_client

    async def upload_code_system(
        self,
        *,
        name: str,
        version: str,
        format: UploadRequestFormat,
        revision: typing.Optional[float] = OMIT,
        file: typing.Optional[str] = OMIT,
        code_col: typing.Optional[str] = OMIT,
        desc_col: typing.Optional[str] = OMIT,
        defn_col: typing.Optional[str] = OMIT,
        codes: typing.Optional[typing.Sequence[CodeResponse]] = OMIT,
        replace: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConstrueUploadCodeSystemResponse:
        """
        Upload a custom medical code system with codes and descriptions for use in code extraction. Requires a paid plan.
        Returns 202 immediately; embedding generation runs asynchronously. Poll
        GET /construe/codes/systems/{codesystem}?version={version} to check when status
        transitions from "processing" to "ready" or "failed".

        Parameters
        ----------
        name : str
            Name of the code system. Names are case-insensitive and stored uppercase.
            Builtin system names (e.g. ICD-10-CM, SNOMED_CT_US_LITE, LOINC, CPT, etc.) are
            reserved and cannot be used for custom uploads; attempts return HTTP 403 Forbidden.

        version : str
            Version of the code system

        format : UploadRequestFormat
            Upload format

        revision : typing.Optional[float]
            Optional revision number

        file : typing.Optional[str]
            The file contents as a base64-encoded string.
            For CSV format, this is the CSV file contents.
            For JSON format, this is a base64-encoded JSON array; prefer using 'codes' instead.

        code_col : typing.Optional[str]
            Column name containing codes (required for CSV format)

        desc_col : typing.Optional[str]
            Column name containing descriptions (required for CSV format)

        defn_col : typing.Optional[str]
            Optional column name containing long definitions (for CSV format)

        codes : typing.Optional[typing.Sequence[CodeResponse]]
            The codes to upload as a JSON array (JSON format only).
            This is the preferred way to upload JSON codes, as it avoids unnecessary base64 encoding.
            If both 'codes' and 'file' are provided, 'codes' takes precedence.

        replace : typing.Optional[bool]
            If true, replaces an existing code system with the same name and version.
            Builtin systems cannot be replaced; attempts to do so return HTTP 403 Forbidden.
            When false (default), uploading a duplicate returns 409 Conflict.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConstrueUploadCodeSystemResponse
            Upload accepted for asynchronous processing

        Examples
        --------
        import asyncio

        from phenoml import AsyncPhenomlClient

        client = AsyncPhenomlClient()


        async def main() -> None:
            await client.construe.upload_code_system(
                name="CUSTOM_CODES",
                version="1.0",
                format="csv",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.upload_code_system(
            name=name,
            version=version,
            format=format,
            revision=revision,
            file=file,
            code_col=code_col,
            desc_col=desc_col,
            defn_col=defn_col,
            codes=codes,
            replace=replace,
            request_options=request_options,
        )
        return _response.data

    async def extract_codes(
        self,
        *,
        text: str,
        system: typing.Optional[ExtractRequestSystem] = OMIT,
        config: typing.Optional[ExtractRequestConfig] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExtractCodesResult:
        """
        Converts natural language text into structured medical codes.

        Usage of CPT is subject to AMA requirements: see PhenoML Terms of Service.

        Parameters
        ----------
        text : str
            Natural language text to extract codes from

        system : typing.Optional[ExtractRequestSystem]

        config : typing.Optional[ExtractRequestConfig]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExtractCodesResult
            Successfully extracted codes

        Examples
        --------
        import asyncio

        from phenoml import AsyncPhenomlClient

        client = AsyncPhenomlClient()


        async def main() -> None:
            await client.construe.extract_codes(
                text="Patient is a 14-year-old female, previously healthy, who is here for evaluation of abnormal renal ultrasound with atrophic right kidney",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.extract_codes(
            text=text, system=system, config=config, request_options=request_options
        )
        return _response.data

    async def list_available_code_systems(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ListCodeSystemsResponse:
        """
        Returns the terminology server's catalog of available code systems, including both built-in standard terminologies and custom uploaded systems.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListCodeSystemsResponse
            List of available code systems

        Examples
        --------
        import asyncio

        from phenoml import AsyncPhenomlClient

        client = AsyncPhenomlClient()


        async def main() -> None:
            await client.construe.list_available_code_systems()


        asyncio.run(main())
        """
        _response = await self._raw_client.list_available_code_systems(request_options=request_options)
        return _response.data

    async def get_code_system_detail(
        self,
        codesystem: str,
        *,
        version: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetCodeSystemDetailResponse:
        """
        Returns full metadata for a single code system, including timestamps and builtin status.

        Parameters
        ----------
        codesystem : str
            Code system name (e.g., "ICD-10-CM", "SNOMED_CT_US_LITE")

        version : typing.Optional[str]
            Specific version of the code system. Required if multiple versions exist.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetCodeSystemDetailResponse
            Code system detail

        Examples
        --------
        import asyncio

        from phenoml import AsyncPhenomlClient

        client = AsyncPhenomlClient()


        async def main() -> None:
            await client.construe.get_code_system_detail(
                codesystem="ICD-10-CM",
                version="2025",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_code_system_detail(
            codesystem, version=version, request_options=request_options
        )
        return _response.data

    async def delete_custom_code_system(
        self,
        codesystem: str,
        *,
        version: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DeleteCodeSystemResponse:
        """
        Deletes a custom (non-builtin) code system and all its codes. Builtin systems cannot be deleted.
        Only available on dedicated instances. Large systems may take up to a minute to delete.

        Parameters
        ----------
        codesystem : str
            Code system name

        version : typing.Optional[str]
            Specific version of the code system. Required if multiple versions exist.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteCodeSystemResponse
            Code system deleted successfully

        Examples
        --------
        import asyncio

        from phenoml import AsyncPhenomlClient

        client = AsyncPhenomlClient()


        async def main() -> None:
            await client.construe.delete_custom_code_system(
                codesystem="CUSTOM_CODES",
                version="version",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete_custom_code_system(
            codesystem, version=version, request_options=request_options
        )
        return _response.data

    async def export_custom_code_system(
        self,
        codesystem: str,
        *,
        version: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExportCodeSystemResponse:
        """
        Exports a custom (non-builtin) code system as a JSON file compatible with the upload format.
        The exported file can be re-uploaded directly via POST /construe/upload with format "json".
        Only available on dedicated instances. Builtin systems cannot be exported.

        Parameters
        ----------
        codesystem : str
            Code system name

        version : typing.Optional[str]
            Specific version of the code system. Required if multiple versions exist.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExportCodeSystemResponse
            Exported code system as JSON file

        Examples
        --------
        import asyncio

        from phenoml import AsyncPhenomlClient

        client = AsyncPhenomlClient()


        async def main() -> None:
            await client.construe.export_custom_code_system(
                codesystem="CUSTOM_CODES",
                version="version",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.export_custom_code_system(
            codesystem, version=version, request_options=request_options
        )
        return _response.data

    async def list_codes_in_a_code_system(
        self,
        codesystem: str,
        *,
        version: typing.Optional[str] = None,
        cursor: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListCodesResponse:
        """
        Returns a paginated list of all codes in the specified code system from the terminology server.

        Usage of CPT is subject to AMA requirements: see PhenoML Terms of Service.

        Parameters
        ----------
        codesystem : str
            Code system name (e.g., "ICD-10-CM", "SNOMED_CT_US_LITE")

        version : typing.Optional[str]
            Specific version of the code system. Required if multiple versions exist.

        cursor : typing.Optional[str]
            Pagination cursor from previous response

        limit : typing.Optional[int]
            Maximum number of codes to return (default 20)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListCodesResponse
            Paginated list of codes

        Examples
        --------
        import asyncio

        from phenoml import AsyncPhenomlClient

        client = AsyncPhenomlClient()


        async def main() -> None:
            await client.construe.list_codes_in_a_code_system(
                codesystem="ICD-10-CM",
                version="2025",
                cursor="cursor",
                limit=1,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list_codes_in_a_code_system(
            codesystem, version=version, cursor=cursor, limit=limit, request_options=request_options
        )
        return _response.data

    async def get_a_specific_code(
        self,
        codesystem: str,
        code_id: str,
        *,
        version: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetCodeResponse:
        """
        Looks up a specific code in the terminology server and returns its details.

        Usage of CPT is subject to AMA requirements: see PhenoML Terms of Service.

        Parameters
        ----------
        codesystem : str
            Code system name

        code_id : str
            The code identifier

        version : typing.Optional[str]
            Specific version of the code system

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetCodeResponse
            Code details

        Examples
        --------
        import asyncio

        from phenoml import AsyncPhenomlClient

        client = AsyncPhenomlClient()


        async def main() -> None:
            await client.construe.get_a_specific_code(
                codesystem="ICD-10-CM",
                code_id="E11.65",
                version="version",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_a_specific_code(
            codesystem, code_id, version=version, request_options=request_options
        )
        return _response.data

    async def semantic_search_embedding_based(
        self,
        codesystem: str,
        *,
        text: str,
        version: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SemanticSearchResponse:
        """
        Performs semantic similarity search using vector embeddings.

        **Availability**: This endpoint works for both **built-in and custom** code systems.

        **When to use**: Best for natural language queries where you want to find conceptually
        related codes, even when different terminology is used. The search understands meaning,
        not just keywords.

        **Examples**:
        - Query "trouble breathing at night" finds codes like "Sleep apnea", "Orthopnea",
          "Nocturnal dyspnea" — semantically related but no exact keyword matches
        - Query "heart problems" finds "Myocardial infarction", "Cardiac arrest", "Arrhythmia"

        **Trade-offs**: Slower than text search (requires embedding generation), but finds
        conceptually similar results that keyword search would miss.

        See also: `/search/text` for faster keyword-based lookup with typo tolerance.

        Usage of CPT is subject to AMA requirements: see PhenoML Terms of Service.

        Parameters
        ----------
        codesystem : str
            Code system name

        text : str
            Natural language text to find semantically similar codes for

        version : typing.Optional[str]
            Specific version of the code system

        limit : typing.Optional[int]
            Maximum number of results (default 10, max 50)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SemanticSearchResponse
            Semantic search results ordered by similarity

        Examples
        --------
        import asyncio

        from phenoml import AsyncPhenomlClient

        client = AsyncPhenomlClient()


        async def main() -> None:
            await client.construe.semantic_search_embedding_based(
                codesystem="ICD-10-CM",
                text="patient has trouble breathing at night and wakes up gasping",
                version="version",
                limit=1,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.semantic_search_embedding_based(
            codesystem, text=text, version=version, limit=limit, request_options=request_options
        )
        return _response.data

    async def submit_feedback_on_extraction_results(
        self,
        *,
        text: str,
        received_result: ExtractCodesResult,
        expected_result: ExtractCodesResult,
        detail: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FeedbackResponse:
        """
        Submits user feedback on results from the Construe extraction endpoint.
        Feedback includes the full extraction result received and the result the user expected.

        Parameters
        ----------
        text : str
            The natural language text that was used for code extraction

        received_result : ExtractCodesResult

        expected_result : ExtractCodesResult

        detail : typing.Optional[str]
            Optional details explaining the feedback

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FeedbackResponse
            Feedback saved successfully

        Examples
        --------
        import asyncio

        from phenoml import AsyncPhenomlClient
        from phenoml.construe import (
            ExtractCodesResult,
            ExtractedCodeResult,
            ExtractRequestSystem,
        )

        client = AsyncPhenomlClient()


        async def main() -> None:
            await client.construe.submit_feedback_on_extraction_results(
                text="Patient has type 2 diabetes with hyperglycemia",
                received_result=ExtractCodesResult(
                    system=ExtractRequestSystem(),
                    codes=[
                        ExtractedCodeResult(
                            code="195967001",
                            description="Asthma",
                            valid=True,
                        )
                    ],
                ),
                expected_result=ExtractCodesResult(
                    system=ExtractRequestSystem(),
                    codes=[
                        ExtractedCodeResult(
                            code="195967001",
                            description="Asthma",
                            valid=True,
                        )
                    ],
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.submit_feedback_on_extraction_results(
            text=text,
            received_result=received_result,
            expected_result=expected_result,
            detail=detail,
            request_options=request_options,
        )
        return _response.data

    async def terminology_server_text_search(
        self,
        codesystem: str,
        *,
        q: str,
        version: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TextSearchResponse:
        """
        Performs fast full-text search over code IDs and descriptions.

        **Availability**: This endpoint is only available for **built-in code systems**.
        Custom code systems uploaded via `/construe/upload` are not indexed for full-text search
        and will return empty results. Use `/search/semantic` to search custom code systems.

        **When to use**: Best for autocomplete UIs, code lookup, or when users know part of
        the code ID or specific keywords. Fast response times suitable for typeahead interfaces.

        **Features**:
        - Substring matching on code IDs (e.g., "11.65" finds "E11.65")
        - Typo tolerance on descriptions (not on code IDs)
        - Fast response times (~10-50ms)

        **Examples**:
        - Query "E11" finds all codes starting with E11 (diabetes codes)
        - Query "diabtes" (typo) still finds "diabetes" codes

        **Trade-offs**: Faster than semantic search, but only matches keywords/substrings.
        Won't find conceptually related codes with different terminology.

        See also: `/search/semantic` for finding conceptually similar codes.

        Usage of CPT is subject to AMA requirements: see PhenoML Terms of Service.

        Parameters
        ----------
        codesystem : str
            Code system name

        q : str
            Search query (searches code IDs and descriptions)

        version : typing.Optional[str]
            Specific version of the code system

        limit : typing.Optional[int]
            Maximum number of results (default 20, max 100)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TextSearchResponse
            Text search results

        Examples
        --------
        import asyncio

        from phenoml import AsyncPhenomlClient

        client = AsyncPhenomlClient()


        async def main() -> None:
            await client.construe.terminology_server_text_search(
                codesystem="ICD-10-CM",
                q="E11.65",
                version="version",
                limit=1,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.terminology_server_text_search(
            codesystem, q=q, version=version, limit=limit, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

AgentChatStreamEventType = typing.Union[
    typing.Literal["message_start", "content_delta", "tool_use", "tool_result", "message_end", "error"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .search_response_resource_type import SearchResponseResourceType


class SearchResponse(UniversalBaseModel):
    resource_type: typing_extensions.Annotated[
        typing.Optional[SearchResponseResourceType],
        FieldMetadata(alias="resourceType"),
        pydantic.Field(alias="resourceType", description="The FHIR resource type identified for the search"),
    ] = None
    search_params: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="searchParams"),
        pydantic.Field(
            alias="searchParams",
            description="FHIR search parameters in standard query string format.\nParameters are formatted according to the FHIR specification with appropriate operators.\nCode parameters are resolved to standard terminology codes (SNOMED CT, LOINC, RxNorm, ICD-10-CM).",
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        AuthMethod,
        ClientSecretAuth,
        FhirProviderAddAuthConfigRequest,
        FhirProviderAddAuthConfigRequest_ClientSecret,
        FhirProviderAddAuthConfigRequest_GoogleHealthcare,
        FhirProviderAddAuthConfigRequest_Jwt,
        FhirProviderAddAuthConfigRequest_None,
        FhirProviderAddAuthConfigRequest_OnBehalfOf,
        FhirProviderAddAuthConfigRequest_TokenPassthrough,
        FhirProviderAuthConfig,
        FhirProviderCreateRequestAuth,
        FhirProviderCreateRequestAuth_ClientSecret,
        FhirProviderCreateRequestAuth_GoogleHealthcare,
        FhirProviderCreateRequestAuth_Jwt,
        FhirProviderCreateRequestAuth_None,
        FhirProviderCreateRequestAuth_OnBehalfOf,
        FhirProviderCreateRequestAuth_TokenPassthrough,
        FhirProviderDeleteResponse,
        FhirProviderListResponse,
        FhirProviderListResponseFhirProvidersItem,
        FhirProviderRemoveAuthConfigResponse,
        FhirProviderResponse,
        FhirProviderResponseData,
        FhirProviderSandboxInfo,
        FhirProviderTemplate,
        FhirQueryResponse,
        FhirQueryResponseData,
        GoogleHealthcareAuth,
        JsonWebKey,
        JwtAuth,
        NoAuth,
        OnBehalfOfAuth,
        Provider,
        Role,
        ServiceAccountKey,
        ServiceAccountMetadata,
        SmartConfiguration,
        TokenPassthroughAuth,
    )
    from .errors import BadRequestError, ForbiddenError, InternalServerError, NotFoundError, UnauthorizedError
_dynamic_imports: typing.Dict[str, str] = {
    "AuthMethod": ".types",
    "BadRequestError": ".errors",
    "ClientSecretAuth": ".types",
    "FhirProviderAddAuthConfigRequest": ".types",
    "FhirProviderAddAuthConfigRequest_ClientSecret": ".types",
    "FhirProviderAddAuthConfigRequest_GoogleHealthcare": ".types",
    "FhirProviderAddAuthConfigRequest_Jwt": ".types",
    "FhirProviderAddAuthConfigRequest_None": ".types",
    "FhirProviderAddAuthConfigRequest_OnBehalfOf": ".types",
    "FhirProviderAddAuthConfigRequest_TokenPassthrough": ".types",
    "FhirProviderAuthConfig": ".types",
    "FhirProviderCreateRequestAuth": ".types",
    "FhirProviderCreateRequestAuth_ClientSecret": ".types",
    "FhirProviderCreateRequestAuth_GoogleHealthcare": ".types",
    "FhirProviderCreateRequestAuth_Jwt": ".types",
    "FhirProviderCreateRequestAuth_None": ".types",
    "FhirProviderCreateRequestAuth_OnBehalfOf": ".types",
    "FhirProviderCreateRequestAuth_TokenPassthrough": ".types",
    "FhirProviderDeleteResponse": ".types",
    "FhirProviderListResponse": ".types",
    "FhirProviderListResponseFhirProvidersItem": ".types",
    "FhirProviderRemoveAuthConfigResponse": ".types",
    "FhirProviderResponse": ".types",
    "FhirProviderResponseData": ".types",
    "FhirProviderSandboxInfo": ".types",
    "FhirProviderTemplate": ".types",
    "FhirQueryResponse": ".types",
    "FhirQueryResponseData": ".types",
    "ForbiddenError": ".errors",
    "GoogleHealthcareAuth": ".types",
    "InternalServerError": ".errors",
    "JsonWebKey": ".types",
    "JwtAuth": ".types",
    "NoAuth": ".types",
    "NotFoundError": ".errors",
    "OnBehalfOfAuth": ".types",
    "Provider": ".types",
    "Role": ".types",
    "ServiceAccountKey": ".types",
    "ServiceAccountMetadata": ".types",
    "SmartConfiguration": ".types",
    "TokenPassthroughAuth": ".types",
    "UnauthorizedError": ".errors",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "AuthMethod",
    "BadRequestError",
    "ClientSecretAuth",
    "FhirProviderAddAuthConfigRequest",
    "FhirProviderAddAuthConfigRequest_ClientSecret",
    "FhirProviderAddAuthConfigRequest_GoogleHealthcare",
    "FhirProviderAddAuthConfigRequest_Jwt",
    "FhirProviderAddAuthConfigRequest_None",
    "FhirProviderAddAuthConfigRequest_OnBehalfOf",
    "FhirProviderAddAuthConfigRequest_TokenPassthrough",
    "FhirProviderAuthConfig",
    "FhirProviderCreateRequestAuth",
    "FhirProviderCreateRequestAuth_ClientSecret",
    "FhirProviderCreateRequestAuth_GoogleHealthcare",
    "FhirProviderCreateRequestAuth_Jwt",
    "FhirProviderCreateRequestAuth_None",
    "FhirProviderCreateRequestAuth_OnBehalfOf",
    "FhirProviderCreateRequestAuth_TokenPassthrough",
    "FhirProviderDeleteResponse",
    "FhirProviderListResponse",
    "FhirProviderListResponseFhirProvidersItem",
    "FhirProviderRemoveAuthConfigResponse",
    "FhirProviderResponse",
    "FhirProviderResponseData",
    "FhirProviderSandboxInfo",
    "FhirProviderTemplate",
    "FhirQueryResponse",
    "FhirQueryResponseData",
    "ForbiddenError",
    "GoogleHealthcareAuth",
    "InternalServerError",
    "JsonWebKey",
    "JwtAuth",
    "NoAuth",
    "NotFoundError",
    "OnBehalfOfAuth",
    "Provider",
    "Role",
    "ServiceAccountKey",
    "ServiceAccountMetadata",
    "SmartConfiguration",
    "TokenPassthroughAuth",
    "UnauthorizedError",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .agent_chat_stream_event_type import AgentChatStreamEventType


class AgentChatStreamEvent(UniversalBaseModel):
    """
    JSON payload sent in the `data:` line of a Server-Sent Event (SSE) frame for streaming chat responses.
    Each SSE frame uses the standard `event:` and `data:` lines; this schema describes the JSON object
    in the `data:` line. The event type is encoded in the top-level `type` property.
    Event types: message_start, content_delta, tool_use, tool_result, message_end, error.
    """

    type: typing.Optional[AgentChatStreamEventType] = pydantic.Field(default=None)
    """
    The event type
    """

    session_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Chat session ID
    """

    content: typing.Optional[str] = pydantic.Field(default=None)
    """
    Incremental text content (present in content_delta events)
    """

    success: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the operation was successful
    """

    message: typing.Optional[str] = pydantic.Field(default=None)
    """
    Status message
    """

    function_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Tool/function name (present in tool_use and tool_result events)
    """

    function_args: typing.Optional[typing.Dict[str, typing.Any]] = pydantic.Field(default=None)
    """
    Tool arguments (present in tool_use events)
    """

    function_result: typing.Optional[typing.Dict[str, typing.Any]] = pydantic.Field(default=None)
    """
    Tool execution result (present in tool_result events)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .auth_method import AuthMethod
    from .client_secret_auth import ClientSecretAuth
    from .fhir_provider_add_auth_config_request import (
        FhirProviderAddAuthConfigRequest,
        FhirProviderAddAuthConfigRequest_ClientSecret,
        FhirProviderAddAuthConfigRequest_GoogleHealthcare,
        FhirProviderAddAuthConfigRequest_Jwt,
        FhirProviderAddAuthConfigRequest_None,
        FhirProviderAddAuthConfigRequest_OnBehalfOf,
        FhirProviderAddAuthConfigRequest_TokenPassthrough,
    )
    from .fhir_provider_auth_config import FhirProviderAuthConfig
    from .fhir_provider_create_request_auth import (
        FhirProviderCreateRequestAuth,
        FhirProviderCreateRequestAuth_ClientSecret,
        FhirProviderCreateRequestAuth_GoogleHealthcare,
        FhirProviderCreateRequestAuth_Jwt,
        FhirProviderCreateRequestAuth_None,
        FhirProviderCreateRequestAuth_OnBehalfOf,
        FhirProviderCreateRequestAuth_TokenPassthrough,
    )
    from .fhir_provider_delete_response import FhirProviderDeleteResponse
    from .fhir_provider_list_response import FhirProviderListResponse
    from .fhir_provider_list_response_fhir_providers_item import FhirProviderListResponseFhirProvidersItem
    from .fhir_provider_remove_auth_config_response import FhirProviderRemoveAuthConfigResponse
    from .fhir_provider_response import FhirProviderResponse
    from .fhir_provider_response_data import FhirProviderResponseData
    from .fhir_provider_sandbox_info import FhirProviderSandboxInfo
    from .fhir_provider_template import FhirProviderTemplate
    from .fhir_query_response import FhirQueryResponse
    from .fhir_query_response_data import FhirQueryResponseData
    from .google_healthcare_auth import GoogleHealthcareAuth
    from .json_web_key import JsonWebKey
    from .jwt_auth import JwtAuth
    from .no_auth import NoAuth
    from .on_behalf_of_auth import OnBehalfOfAuth
    from .provider import Provider
    from .role import Role
    from .service_account_key import ServiceAccountKey
    from .service_account_metadata import ServiceAccountMetadata
    from .smart_configuration import SmartConfiguration
    from .token_passthrough_auth import TokenPassthroughAuth
_dynamic_imports: typing.Dict[str, str] = {
    "AuthMethod": ".auth_method",
    "ClientSecretAuth": ".client_secret_auth",
    "FhirProviderAddAuthConfigRequest": ".fhir_provider_add_auth_config_request",
    "FhirProviderAddAuthConfigRequest_ClientSecret": ".fhir_provider_add_auth_config_request",
    "FhirProviderAddAuthConfigRequest_GoogleHealthcare": ".fhir_provider_add_auth_config_request",
    "FhirProviderAddAuthConfigRequest_Jwt": ".fhir_provider_add_auth_config_request",
    "FhirProviderAddAuthConfigRequest_None": ".fhir_provider_add_auth_config_request",
    "FhirProviderAddAuthConfigRequest_OnBehalfOf": ".fhir_provider_add_auth_config_request",
    "FhirProviderAddAuthConfigRequest_TokenPassthrough": ".fhir_provider_add_auth_config_request",
    "FhirProviderAuthConfig": ".fhir_provider_auth_config",
    "FhirProviderCreateRequestAuth": ".fhir_provider_create_request_auth",
    "FhirProviderCreateRequestAuth_ClientSecret": ".fhir_provider_create_request_auth",
    "FhirProviderCreateRequestAuth_GoogleHealthcare": ".fhir_provider_create_request_auth",
    "FhirProviderCreateRequestAuth_Jwt": ".fhir_provider_create_request_auth",
    "FhirProviderCreateRequestAuth_None": ".fhir_provider_create_request_auth",
    "FhirProviderCreateRequestAuth_OnBehalfOf": ".fhir_provider_create_request_auth",
    "FhirProviderCreateRequestAuth_TokenPassthrough": ".fhir_provider_create_request_auth",
    "FhirProviderDeleteResponse": ".fhir_provider_delete_response",
    "FhirProviderListResponse": ".fhir_provider_list_response",
    "FhirProviderListResponseFhirProvidersItem": ".fhir_provider_list_response_fhir_providers_item",
    "FhirProviderRemoveAuthConfigResponse": ".fhir_provider_remove_auth_config_response",
    "FhirProviderResponse": ".fhir_provider_response",
    "FhirProviderResponseData": ".fhir_provider_response_data",
    "FhirProviderSandboxInfo": ".fhir_provider_sandbox_info",
    "FhirProviderTemplate": ".fhir_provider_template",
    "FhirQueryResponse": ".fhir_query_response",
    "FhirQueryResponseData": ".fhir_query_response_data",
    "GoogleHealthcareAuth": ".google_healthcare_auth",
    "JsonWebKey": ".json_web_key",
    "JwtAuth": ".jwt_auth",
    "NoAuth": ".no_auth",
    "OnBehalfOfAuth": ".on_behalf_of_auth",
    "Provider": ".provider",
    "Role": ".role",
    "ServiceAccountKey": ".service_account_key",
    "ServiceAccountMetadata": ".service_account_metadata",
    "SmartConfiguration": ".smart_configuration",
    "TokenPassthroughAuth": ".token_passthrough_auth",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "AuthMethod",
    "ClientSecretAuth",
    "FhirProviderAddAuthConfigRequest",
    "FhirProviderAddAuthConfigRequest_ClientSecret",
    "FhirProviderAddAuthConfigRequest_GoogleHealthcare",
    "FhirProviderAddAuthConfigRequest_Jwt",
    "FhirProviderAddAuthConfigRequest_None",
    "FhirProviderAddAuthConfigRequest_OnBehalfOf",
    "FhirProviderAddAuthConfigRequest_TokenPassthrough",
    "FhirProviderAuthConfig",
    "FhirProviderCreateRequestAuth",
    "FhirProviderCreateRequestAuth_ClientSecret",
    "FhirProviderCreateRequestAuth_GoogleHealthcare",
    "FhirProviderCreateRequestAuth_Jwt",
    "FhirProviderCreateRequestAuth_None",
    "FhirProviderCreateRequestAuth_OnBehalfOf",
    "FhirProviderCreateRequestAuth_TokenPassthrough",
    "FhirProviderDeleteResponse",
    "FhirProviderListResponse",
    "FhirProviderListResponseFhirProvidersItem",
    "FhirProviderRemoveAuthConfigResponse",
    "FhirProviderResponse",
    "FhirProviderResponseData",
    "FhirProviderSandboxInfo",
    "FhirProviderTemplate",
    "FhirQueryResponse",
    "FhirQueryResponseData",
    "GoogleHealthcareAuth",
    "JsonWebKey",
    "JwtAuth",
    "NoAuth",
    "OnBehalfOfAuth",
    "Provider",
    "Role",
    "ServiceAccountKey",
    "ServiceAccountMetadata",
    "SmartConfiguration",
    "TokenPassthroughAuth",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        CreateMultiResponse,
        CreateMultiResponseBundle,
        CreateMultiResponseBundleEntryItem,
        CreateMultiResponseBundleEntryItemRequest,
        CreateMultiResponseResourcesItem,
        CreateRequestResource,
        FhirResource,
        Lang2FhirUploadProfileResponse,
        SearchResponse,
        SearchResponseResourceType,
    )
    from .errors import (
        BadRequestError,
        FailedDependencyError,
        ForbiddenError,
        InternalServerError,
        UnauthorizedError,
        UnprocessableEntityError,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "BadRequestError": ".errors",
    "CreateMultiResponse": ".types",
    "CreateMultiResponseBundle": ".types",
    "CreateMultiResponseBundleEntryItem": ".types",
    "CreateMultiResponseBundleEntryItemRequest": ".types",
    "CreateMultiResponseResourcesItem": ".types",
    "CreateRequestResource": ".types",
    "FailedDependencyError": ".errors",
    "FhirResource": ".types",
    "ForbiddenError": ".errors",
    "InternalServerError": ".errors",
    "Lang2FhirUploadProfileResponse": ".types",
    "SearchResponse": ".types",
    "SearchResponseResourceType": ".types",
    "UnauthorizedError": ".errors",
    "UnprocessableEntityError": ".errors",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "BadRequestError",
    "CreateMultiResponse",
    "CreateMultiResponseBundle",
    "CreateMultiResponseBundleEntryItem",
    "CreateMultiResponseBundleEntryItemRequest",
    "CreateMultiResponseResourcesItem",
    "CreateRequestResource",
    "FailedDependencyError",
    "FhirResource",
    "ForbiddenError",
    "InternalServerError",
    "Lang2FhirUploadProfileResponse",
    "SearchResponse",
    "SearchResponseResourceType",
    "UnauthorizedError",
    "UnprocessableEntityError",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        CreateSummaryRequestFhirResources,
        CreateSummaryRequestMode,
        CreateSummaryResponse,
        CreateSummaryTemplateResponse,
        FhirBundle,
        FhirBundleEntryItem,
        FhirResource,
        SummaryDeleteTemplateResponse,
        SummaryGetTemplateResponse,
        SummaryListTemplatesResponse,
        SummaryTemplate,
        SummaryUpdateTemplateResponse,
    )
    from .errors import BadRequestError, ForbiddenError, InternalServerError, NotFoundError, UnauthorizedError
_dynamic_imports: typing.Dict[str, str] = {
    "BadRequestError": ".errors",
    "CreateSummaryRequestFhirResources": ".types",
    "CreateSummaryRequestMode": ".types",
    "CreateSummaryResponse": ".types",
    "CreateSummaryTemplateResponse": ".types",
    "FhirBundle": ".types",
    "FhirBundleEntryItem": ".types",
    "FhirResource": ".types",
    "ForbiddenError": ".errors",
    "InternalServerError": ".errors",
    "NotFoundError": ".errors",
    "SummaryDeleteTemplateResponse": ".types",
    "SummaryGetTemplateResponse": ".types",
    "SummaryListTemplatesResponse": ".types",
    "SummaryTemplate": ".types",
    "SummaryUpdateTemplateResponse": ".types",
    "UnauthorizedError": ".errors",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "BadRequestError",
    "CreateSummaryRequestFhirResources",
    "CreateSummaryRequestMode",
    "CreateSummaryResponse",
    "CreateSummaryTemplateResponse",
    "FhirBundle",
    "FhirBundleEntryItem",
    "FhirResource",
    "ForbiddenError",
    "InternalServerError",
    "NotFoundError",
    "SummaryDeleteTemplateResponse",
    "SummaryGetTemplateResponse",
    "SummaryListTemplatesResponse",
    "SummaryTemplate",
    "SummaryUpdateTemplateResponse",
    "UnauthorizedError",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class CreateMultiResponseResourcesItem(UniversalBaseModel):
    temp_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="tempId"),
        pydantic.Field(alias="tempId", description="Temporary UUID for the resource"),
    ] = None
    resource_type: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="resourceType"),
        pydantic.Field(alias="resourceType", description="FHIR resource type"),
    ] = None
    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Text excerpt this resource was extracted from
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        CreateWorkflowRequestFhirProviderId,
        CreateWorkflowResponse,
        DecisionNodeDefinition,
        ExecuteWorkflowResponse,
        ExecuteWorkflowResponseResults,
        Lang2FhirCreateDefinition,
        Lang2FhirSearchDefinition,
        ListWorkflowsResponse,
        StepOperation,
        SubWorkflowDefinition,
        UpdateWorkflowRequestFhirProviderId,
        WorkflowConfig,
        WorkflowDefinition,
        WorkflowGraph,
        WorkflowResponse,
        WorkflowResponseGraph,
        WorkflowStep,
        WorkflowStepSummary,
        WorkflowStepSummaryType,
        WorkflowStepType,
        WorkflowsDeleteResponse,
        WorkflowsGetResponse,
        WorkflowsUpdateResponse,
    )
    from .errors import BadRequestError, ForbiddenError, InternalServerError, NotFoundError, UnauthorizedError
_dynamic_imports: typing.Dict[str, str] = {
    "BadRequestError": ".errors",
    "CreateWorkflowRequestFhirProviderId": ".types",
    "CreateWorkflowResponse": ".types",
    "DecisionNodeDefinition": ".types",
    "ExecuteWorkflowResponse": ".types",
    "ExecuteWorkflowResponseResults": ".types",
    "ForbiddenError": ".errors",
    "InternalServerError": ".errors",
    "Lang2FhirCreateDefinition": ".types",
    "Lang2FhirSearchDefinition": ".types",
    "ListWorkflowsResponse": ".types",
    "NotFoundError": ".errors",
    "StepOperation": ".types",
    "SubWorkflowDefinition": ".types",
    "UnauthorizedError": ".errors",
    "UpdateWorkflowRequestFhirProviderId": ".types",
    "WorkflowConfig": ".types",
    "WorkflowDefinition": ".types",
    "WorkflowGraph": ".types",
    "WorkflowResponse": ".types",
    "WorkflowResponseGraph": ".types",
    "WorkflowStep": ".types",
    "WorkflowStepSummary": ".types",
    "WorkflowStepSummaryType": ".types",
    "WorkflowStepType": ".types",
    "WorkflowsDeleteResponse": ".types",
    "WorkflowsGetResponse": ".types",
    "WorkflowsUpdateResponse": ".types",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "BadRequestError",
    "CreateWorkflowRequestFhirProviderId",
    "CreateWorkflowResponse",
    "DecisionNodeDefinition",
    "ExecuteWorkflowResponse",
    "ExecuteWorkflowResponseResults",
    "ForbiddenError",
    "InternalServerError",
    "Lang2FhirCreateDefinition",
    "Lang2FhirSearchDefinition",
    "ListWorkflowsResponse",
    "NotFoundError",
    "StepOperation",
    "SubWorkflowDefinition",
    "UnauthorizedError",
    "UpdateWorkflowRequestFhirProviderId",
    "WorkflowConfig",
    "WorkflowDefinition",
    "WorkflowGraph",
    "WorkflowResponse",
    "WorkflowResponseGraph",
    "WorkflowStep",
    "WorkflowStepSummary",
    "WorkflowStepSummaryType",
    "WorkflowStepType",
    "WorkflowsDeleteResponse",
    "WorkflowsGetResponse",
    "WorkflowsUpdateResponse",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class Lang2FhirUploadProfileResponse(UniversalBaseModel):
    message: typing.Optional[str] = None
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The lowercase StructureDefinition id, used as the profile's unique identifier and lookup key. Pass this value as the `resource` parameter to `/lang2fhir/create` or `/lang2fhir/profile/json/:version/:resource` to use this profile.
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The FHIR resource type from the StructureDefinition
    """

    url: typing.Optional[str] = pydantic.Field(default=None)
    """
    The canonical URL from the StructureDefinition
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class CodeSystemDetails(UniversalBaseModel):
    name: str = pydantic.Field()
    """
    Code system name
    """

    version: str = pydantic.Field()
    """
    Code system version
    """

    code_count: int = pydantic.Field()
    """
    Total number of codes in the system
    """

    builtin: bool = pydantic.Field()
    """
    Whether this is a built-in system (vs custom uploaded)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from . import agent, authtoken, cohort, construe, fhir, fhir_provider, lang2fhir, summary, tools, workflows
    from .client import AsyncPhenomlClient, PhenomlClient
    from .environment import PhenomlClientEnvironment
    from .version import __version__
_dynamic_imports: typing.Dict[str, str] = {
    "AsyncPhenomlClient": ".client",
    "PhenomlClient": ".client",
    "PhenomlClientEnvironment": ".environment",
    "__version__": ".version",
    "agent": ".agent",
    "authtoken": ".authtoken",
    "cohort": ".cohort",
    "construe": ".construe",
    "fhir": ".fhir",
    "fhir_provider": ".fhir_provider",
    "lang2fhir": ".lang2fhir",
    "summary": ".summary",
    "tools": ".tools",
    "workflows": ".workflows",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "AsyncPhenomlClient",
    "PhenomlClient",
    "PhenomlClientEnvironment",
    "__version__",
    "agent",
    "authtoken",
    "cohort",
    "construe",
    "fhir",
    "fhir_provider",
    "lang2fhir",
    "summary",
    "tools",
    "workflows",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .create_multi_response import CreateMultiResponse
    from .create_multi_response_bundle import CreateMultiResponseBundle
    from .create_multi_response_bundle_entry_item import CreateMultiResponseBundleEntryItem
    from .create_multi_response_bundle_entry_item_request import CreateMultiResponseBundleEntryItemRequest
    from .create_multi_response_resources_item import CreateMultiResponseResourcesItem
    from .create_request_resource import CreateRequestResource
    from .fhir_resource import FhirResource
    from .lang2fhir_upload_profile_response import Lang2FhirUploadProfileResponse
    from .search_response import SearchResponse
    from .search_response_resource_type import SearchResponseResourceType
_dynamic_imports: typing.Dict[str, str] = {
    "CreateMultiResponse": ".create_multi_response",
    "CreateMultiResponseBundle": ".create_multi_response_bundle",
    "CreateMultiResponseBundleEntryItem": ".create_multi_response_bundle_entry_item",
    "CreateMultiResponseBundleEntryItemRequest": ".create_multi_response_bundle_entry_item_request",
    "CreateMultiResponseResourcesItem": ".create_multi_response_resources_item",
    "CreateRequestResource": ".create_request_resource",
    "FhirResource": ".fhir_resource",
    "Lang2FhirUploadProfileResponse": ".lang2fhir_upload_profile_response",
    "SearchResponse": ".search_response",
    "SearchResponseResourceType": ".search_response_resource_type",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "CreateMultiResponse",
    "CreateMultiResponseBundle",
    "CreateMultiResponseBundleEntryItem",
    "CreateMultiResponseBundleEntryItemRequest",
    "CreateMultiResponseResourcesItem",
    "CreateRequestResource",
    "FhirResource",
    "Lang2FhirUploadProfileResponse",
    "SearchResponse",
    "SearchResponseResourceType",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        AgentChatResponse,
        AgentChatStreamEvent,
        AgentChatStreamEventType,
        AgentCreateRequest,
        AgentCreateRequestProvider,
        AgentDeleteResponse,
        AgentGetChatMessagesRequestOrder,
        AgentGetChatMessagesRequestRole,
        AgentGetChatMessagesResponse,
        AgentListResponse,
        AgentPromptsResponse,
        AgentResponse,
        AgentTemplate,
        AgentTemplateProvider,
        ChatMessageTemplate,
        ChatMessageTemplateRole,
        ChatSessionTemplate,
        JsonPatch,
        JsonPatchOperation,
        JsonPatchOperationOp,
        PromptTemplate,
        SuccessResponse,
    )
    from .errors import BadRequestError, ForbiddenError, InternalServerError, NotFoundError, UnauthorizedError
    from . import prompts
    from .prompts import PromptsDeleteResponse, PromptsListResponse
_dynamic_imports: typing.Dict[str, str] = {
    "AgentChatResponse": ".types",
    "AgentChatStreamEvent": ".types",
    "AgentChatStreamEventType": ".types",
    "AgentCreateRequest": ".types",
    "AgentCreateRequestProvider": ".types",
    "AgentDeleteResponse": ".types",
    "AgentGetChatMessagesRequestOrder": ".types",
    "AgentGetChatMessagesRequestRole": ".types",
    "AgentGetChatMessagesResponse": ".types",
    "AgentListResponse": ".types",
    "AgentPromptsResponse": ".types",
    "AgentResponse": ".types",
    "AgentTemplate": ".types",
    "AgentTemplateProvider": ".types",
    "BadRequestError": ".errors",
    "ChatMessageTemplate": ".types",
    "ChatMessageTemplateRole": ".types",
    "ChatSessionTemplate": ".types",
    "ForbiddenError": ".errors",
    "InternalServerError": ".errors",
    "JsonPatch": ".types",
    "JsonPatchOperation": ".types",
    "JsonPatchOperationOp": ".types",
    "NotFoundError": ".errors",
    "PromptTemplate": ".types",
    "PromptsDeleteResponse": ".prompts",
    "PromptsListResponse": ".prompts",
    "SuccessResponse": ".types",
    "UnauthorizedError": ".errors",
    "prompts": ".prompts",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "AgentChatResponse",
    "AgentChatStreamEvent",
    "AgentChatStreamEventType",
    "AgentCreateRequest",
    "AgentCreateRequestProvider",
    "AgentDeleteResponse",
    "AgentGetChatMessagesRequestOrder",
    "AgentGetChatMessagesRequestRole",
    "AgentGetChatMessagesResponse",
    "AgentListResponse",
    "AgentPromptsResponse",
    "AgentResponse",
    "AgentTemplate",
    "AgentTemplateProvider",
    "BadRequestError",
    "ChatMessageTemplate",
    "ChatMessageTemplateRole",
    "ChatSessionTemplate",
    "ForbiddenError",
    "InternalServerError",
    "JsonPatch",
    "JsonPatchOperation",
    "JsonPatchOperationOp",
    "NotFoundError",
    "PromptTemplate",
    "PromptsDeleteResponse",
    "PromptsListResponse",
    "SuccessResponse",
    "UnauthorizedError",
    "prompts",
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .raw_client import AsyncRawAuthtokenClient, RawAuthtokenClient

if typing.TYPE_CHECKING:
    from .auth.client import AsyncAuthClient, AuthClient


class AuthtokenClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawAuthtokenClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._auth: typing.Optional[AuthClient] = None

    @property
    def with_raw_response(self) -> RawAuthtokenClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawAuthtokenClient
        """
        return self._raw_client

    @property
    def auth(self):
        if self._auth is None:
            from .auth.client import AuthClient  # noqa: E402

            self._auth = AuthClient(client_wrapper=self._client_wrapper)
        return self._auth


class AsyncAuthtokenClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawAuthtokenClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._auth: typing.Optional[AsyncAuthClient] = None

    @property
    def with_raw_response(self) -> AsyncRawAuthtokenClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawAuthtokenClient
        """
        return self._raw_client

    @property
    def auth(self):
        if self._auth is None:
            from .auth.client import AsyncAuthClient  # noqa: E402

            self._auth = AsyncAuthClient(client_wrapper=self._client_wrapper)
        return self._auth



# This file was auto-generated by Fern from our API Definition.

import typing

Provider = typing.Union[
    typing.Literal[
        "athenahealth",
        "canvas",
        "cerner",
        "elation",
        "epic",
        "google_healthcare",
        "hapi",
        "medplum",
        "phenostore",
        "sandbox",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .create_workflow_request_fhir_provider_id import CreateWorkflowRequestFhirProviderId
    from .create_workflow_response import CreateWorkflowResponse
    from .decision_node_definition import DecisionNodeDefinition
    from .execute_workflow_response import ExecuteWorkflowResponse
    from .execute_workflow_response_results import ExecuteWorkflowResponseResults
    from .lang2fhir_create_definition import Lang2FhirCreateDefinition
    from .lang2fhir_search_definition import Lang2FhirSearchDefinition
    from .list_workflows_response import ListWorkflowsResponse
    from .step_operation import StepOperation
    from .sub_workflow_definition import SubWorkflowDefinition
    from .update_workflow_request_fhir_provider_id import UpdateWorkflowRequestFhirProviderId
    from .workflow_config import WorkflowConfig
    from .workflow_definition import WorkflowDefinition
    from .workflow_graph import WorkflowGraph
    from .workflow_response import WorkflowResponse
    from .workflow_response_graph import WorkflowResponseGraph
    from .workflow_step import WorkflowStep
    from .workflow_step_summary import WorkflowStepSummary
    from .workflow_step_summary_type import WorkflowStepSummaryType
    from .workflow_step_type import WorkflowStepType
    from .workflows_delete_response import WorkflowsDeleteResponse
    from .workflows_get_response import WorkflowsGetResponse
    from .workflows_update_response import WorkflowsUpdateResponse
_dynamic_imports: typing.Dict[str, str] = {
    "CreateWorkflowRequestFhirProviderId": ".create_workflow_request_fhir_provider_id",
    "CreateWorkflowResponse": ".create_workflow_response",
    "DecisionNodeDefinition": ".decision_node_definition",
    "ExecuteWorkflowResponse": ".execute_workflow_response",
    "ExecuteWorkflowResponseResults": ".execute_workflow_response_results",
    "Lang2FhirCreateDefinition": ".lang2fhir_create_definition",
    "Lang2FhirSearchDefinition": ".lang2fhir_search_definition",
    "ListWorkflowsResponse": ".list_workflows_response",
    "StepOperation": ".step_operation",
    "SubWorkflowDefinition": ".sub_workflow_definition",
    "UpdateWorkflowRequestFhirProviderId": ".update_workflow_request_fhir_provider_id",
    "WorkflowConfig": ".workflow_config",
    "WorkflowDefinition": ".workflow_definition",
    "WorkflowGraph": ".workflow_graph",
    "WorkflowResponse": ".workflow_response",
    "WorkflowResponseGraph": ".workflow_response_graph",
    "WorkflowStep": ".workflow_step",
    "WorkflowStepSummary": ".workflow_step_summary",
    "WorkflowStepSummaryType": ".workflow_step_summary_type",
    "WorkflowStepType": ".workflow_step_type",
    "WorkflowsDeleteResponse": ".workflows_delete_response",
    "WorkflowsGetResponse": ".workflows_get_response",
    "WorkflowsUpdateResponse": ".workflows_update_response",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "CreateWorkflowRequestFhirProviderId",
    "CreateWorkflowResponse",
    "DecisionNodeDefinition",
    "ExecuteWorkflowResponse",
    "ExecuteWorkflowResponseResults",
    "Lang2FhirCreateDefinition",
    "Lang2FhirSearchDefinition",
    "ListWorkflowsResponse",
    "StepOperation",
    "SubWorkflowDefinition",
    "UpdateWorkflowRequestFhirProviderId",
    "WorkflowConfig",
    "WorkflowDefinition",
    "WorkflowGraph",
    "WorkflowResponse",
    "WorkflowResponseGraph",
    "WorkflowStep",
    "WorkflowStepSummary",
    "WorkflowStepSummaryType",
    "WorkflowStepType",
    "WorkflowsDeleteResponse",
    "WorkflowsGetResponse",
    "WorkflowsUpdateResponse",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .o_auth_error_error import OAuthErrorError


class OAuthError(UniversalBaseModel):
    """
    OAuth 2.0 error response (RFC 6749 §5.2)
    """

    error: OAuthErrorError = pydantic.Field()
    """
    Error code
    """

    error_description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Human-readable error description
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .role import Role


class OnBehalfOfAuth(UniversalBaseModel):
    client_id: str = pydantic.Field()
    """
    OAuth client ID
    """

    client_secret: str = pydantic.Field()
    """
    OAuth client secret
    """

    role: typing.Optional[Role] = None
    scopes: typing.Optional[str] = pydantic.Field(default=None)
    """
    OAuth scopes to request. Cannot be specified with role. If neither role nor scopes are specified, the provider-specific default role will be used.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .citation import Citation
    from .code_response import CodeResponse
    from .code_system_details import CodeSystemDetails
    from .code_system_info import CodeSystemInfo
    from .construe_upload_code_system_response import ConstrueUploadCodeSystemResponse
    from .delete_code_system_response import DeleteCodeSystemResponse
    from .export_code_system_response import ExportCodeSystemResponse
    from .extract_codes_result import ExtractCodesResult
    from .extract_request_config import ExtractRequestConfig
    from .extract_request_config_chunking_method import ExtractRequestConfigChunkingMethod
    from .extract_request_config_validation_method import ExtractRequestConfigValidationMethod
    from .extract_request_system import ExtractRequestSystem
    from .extracted_code_result import ExtractedCodeResult
    from .feedback_response import FeedbackResponse
    from .get_code_response import GetCodeResponse
    from .get_code_system_detail_response import GetCodeSystemDetailResponse
    from .get_code_system_detail_response_status import GetCodeSystemDetailResponseStatus
    from .list_code_systems_response import ListCodeSystemsResponse
    from .list_codes_response import ListCodesResponse
    from .semantic_search_response import SemanticSearchResponse
    from .semantic_search_result import SemanticSearchResult
    from .text_search_response import TextSearchResponse
    from .text_search_result import TextSearchResult
    from .upload_request_format import UploadRequestFormat
_dynamic_imports: typing.Dict[str, str] = {
    "Citation": ".citation",
    "CodeResponse": ".code_response",
    "CodeSystemDetails": ".code_system_details",
    "CodeSystemInfo": ".code_system_info",
    "ConstrueUploadCodeSystemResponse": ".construe_upload_code_system_response",
    "DeleteCodeSystemResponse": ".delete_code_system_response",
    "ExportCodeSystemResponse": ".export_code_system_response",
    "ExtractCodesResult": ".extract_codes_result",
    "ExtractRequestConfig": ".extract_request_config",
    "ExtractRequestConfigChunkingMethod": ".extract_request_config_chunking_method",
    "ExtractRequestConfigValidationMethod": ".extract_request_config_validation_method",
    "ExtractRequestSystem": ".extract_request_system",
    "ExtractedCodeResult": ".extracted_code_result",
    "FeedbackResponse": ".feedback_response",
    "GetCodeResponse": ".get_code_response",
    "GetCodeSystemDetailResponse": ".get_code_system_detail_response",
    "GetCodeSystemDetailResponseStatus": ".get_code_system_detail_response_status",
    "ListCodeSystemsResponse": ".list_code_systems_response",
    "ListCodesResponse": ".list_codes_response",
    "SemanticSearchResponse": ".semantic_search_response",
    "SemanticSearchResult": ".semantic_search_result",
    "TextSearchResponse": ".text_search_response",
    "TextSearchResult": ".text_search_result",
    "UploadRequestFormat": ".upload_request_format",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "Citation",
    "CodeResponse",
    "CodeSystemDetails",
    "CodeSystemInfo",
    "ConstrueUploadCodeSystemResponse",
    "DeleteCodeSystemResponse",
    "ExportCodeSystemResponse",
    "ExtractCodesResult",
    "ExtractRequestConfig",
    "ExtractRequestConfigChunkingMethod",
    "ExtractRequestConfigValidationMethod",
    "ExtractRequestSystem",
    "ExtractedCodeResult",
    "FeedbackResponse",
    "GetCodeResponse",
    "GetCodeSystemDetailResponse",
    "GetCodeSystemDetailResponseStatus",
    "ListCodeSystemsResponse",
    "ListCodesResponse",
    "SemanticSearchResponse",
    "SemanticSearchResult",
    "TextSearchResponse",
    "TextSearchResult",
    "UploadRequestFormat",
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import datetime as dt
import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .role import Role
from .service_account_key import ServiceAccountKey


class FhirProviderCreateRequestAuth_Jwt(UniversalBaseModel):
    auth_method: typing.Literal["jwt"] = "jwt"
    client_id: str
    credential_expiry: typing.Optional[dt.datetime] = None
    role: typing.Optional[Role] = None
    scopes: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class FhirProviderCreateRequestAuth_ClientSecret(UniversalBaseModel):
    auth_method: typing.Literal["client_secret"] = "client_secret"
    client_id: str
    client_secret: str
    role: typing.Optional[Role] = None
    scopes: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class FhirProviderCreateRequestAuth_OnBehalfOf(UniversalBaseModel):
    auth_method: typing.Literal["on_behalf_of"] = "on_behalf_of"
    client_id: str
    client_secret: str
    role: typing.Optional[Role] = None
    scopes: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class FhirProviderCreateRequestAuth_GoogleHealthcare(UniversalBaseModel):
    auth_method: typing.Literal["google_healthcare"] = "google_healthcare"
    service_account_key: ServiceAccountKey

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class FhirProviderCreateRequestAuth_TokenPassthrough(UniversalBaseModel):
    auth_method: typing.Literal["token_passthrough"] = "token_passthrough"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class FhirProviderCreateRequestAuth_None(UniversalBaseModel):
    auth_method: typing.Literal["none"] = "none"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


FhirProviderCreateRequestAuth = typing_extensions.Annotated[
    typing.Union[
        FhirProviderCreateRequestAuth_Jwt,
        FhirProviderCreateRequestAuth_ClientSecret,
        FhirProviderCreateRequestAuth_OnBehalfOf,
        FhirProviderCreateRequestAuth_GoogleHealthcare,
        FhirProviderCreateRequestAuth_TokenPassthrough,
        FhirProviderCreateRequestAuth_None,
    ],
    pydantic.Field(discriminator="auth_method"),
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .create_summary_request_fhir_resources import CreateSummaryRequestFhirResources
    from .create_summary_request_mode import CreateSummaryRequestMode
    from .create_summary_response import CreateSummaryResponse
    from .create_summary_template_response import CreateSummaryTemplateResponse
    from .fhir_bundle import FhirBundle
    from .fhir_bundle_entry_item import FhirBundleEntryItem
    from .fhir_resource import FhirResource
    from .summary_delete_template_response import SummaryDeleteTemplateResponse
    from .summary_get_template_response import SummaryGetTemplateResponse
    from .summary_list_templates_response import SummaryListTemplatesResponse
    from .summary_template import SummaryTemplate
    from .summary_update_template_response import SummaryUpdateTemplateResponse
_dynamic_imports: typing.Dict[str, str] = {
    "CreateSummaryRequestFhirResources": ".create_summary_request_fhir_resources",
    "CreateSummaryRequestMode": ".create_summary_request_mode",
    "CreateSummaryResponse": ".create_summary_response",
    "CreateSummaryTemplateResponse": ".create_summary_template_response",
    "FhirBundle": ".fhir_bundle",
    "FhirBundleEntryItem": ".fhir_bundle_entry_item",
    "FhirResource": ".fhir_resource",
    "SummaryDeleteTemplateResponse": ".summary_delete_template_response",
    "SummaryGetTemplateResponse": ".summary_get_template_response",
    "SummaryListTemplatesResponse": ".summary_list_templates_response",
    "SummaryTemplate": ".summary_template",
    "SummaryUpdateTemplateResponse": ".summary_update_template_response",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "CreateSummaryRequestFhirResources",
    "CreateSummaryRequestMode",
    "CreateSummaryResponse",
    "CreateSummaryTemplateResponse",
    "FhirBundle",
    "FhirBundleEntryItem",
    "FhirResource",
    "SummaryDeleteTemplateResponse",
    "SummaryGetTemplateResponse",
    "SummaryListTemplatesResponse",
    "SummaryTemplate",
    "SummaryUpdateTemplateResponse",
]



# This file was auto-generated by Fern from our API Definition.

import typing

OAuthErrorError = typing.Union[
    typing.Literal["invalid_request", "invalid_client", "unsupported_grant_type", "server_error"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        Citation,
        CodeResponse,
        CodeSystemDetails,
        CodeSystemInfo,
        ConstrueUploadCodeSystemResponse,
        DeleteCodeSystemResponse,
        ExportCodeSystemResponse,
        ExtractCodesResult,
        ExtractRequestConfig,
        ExtractRequestConfigChunkingMethod,
        ExtractRequestConfigValidationMethod,
        ExtractRequestSystem,
        ExtractedCodeResult,
        FeedbackResponse,
        GetCodeResponse,
        GetCodeSystemDetailResponse,
        GetCodeSystemDetailResponseStatus,
        ListCodeSystemsResponse,
        ListCodesResponse,
        SemanticSearchResponse,
        SemanticSearchResult,
        TextSearchResponse,
        TextSearchResult,
        UploadRequestFormat,
    )
    from .errors import (
        BadRequestError,
        ConflictError,
        FailedDependencyError,
        ForbiddenError,
        GatewayTimeoutError,
        InternalServerError,
        NotFoundError,
        NotImplementedError,
        ServiceUnavailableError,
        UnauthorizedError,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "BadRequestError": ".errors",
    "Citation": ".types",
    "CodeResponse": ".types",
    "CodeSystemDetails": ".types",
    "CodeSystemInfo": ".types",
    "ConflictError": ".errors",
    "ConstrueUploadCodeSystemResponse": ".types",
    "DeleteCodeSystemResponse": ".types",
    "ExportCodeSystemResponse": ".types",
    "ExtractCodesResult": ".types",
    "ExtractRequestConfig": ".types",
    "ExtractRequestConfigChunkingMethod": ".types",
    "ExtractRequestConfigValidationMethod": ".types",
    "ExtractRequestSystem": ".types",
    "ExtractedCodeResult": ".types",
    "FailedDependencyError": ".errors",
    "FeedbackResponse": ".types",
    "ForbiddenError": ".errors",
    "GatewayTimeoutError": ".errors",
    "GetCodeResponse": ".types",
    "GetCodeSystemDetailResponse": ".types",
    "GetCodeSystemDetailResponseStatus": ".types",
    "InternalServerError": ".errors",
    "ListCodeSystemsResponse": ".types",
    "ListCodesResponse": ".types",
    "NotFoundError": ".errors",
    "NotImplementedError": ".errors",
    "SemanticSearchResponse": ".types",
    "SemanticSearchResult": ".types",
    "ServiceUnavailableError": ".errors",
    "TextSearchResponse": ".types",
    "TextSearchResult": ".types",
    "UnauthorizedError": ".errors",
    "UploadRequestFormat": ".types",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "BadRequestError",
    "Citation",
    "CodeResponse",
    "CodeSystemDetails",
    "CodeSystemInfo",
    "ConflictError",
    "ConstrueUploadCodeSystemResponse",
    "DeleteCodeSystemResponse",
    "ExportCodeSystemResponse",
    "ExtractCodesResult",
    "ExtractRequestConfig",
    "ExtractRequestConfigChunkingMethod",
    "ExtractRequestConfigValidationMethod",
    "ExtractRequestSystem",
    "ExtractedCodeResult",
    "FailedDependencyError",
    "FeedbackResponse",
    "ForbiddenError",
    "GatewayTimeoutError",
    "GetCodeResponse",
    "GetCodeSystemDetailResponse",
    "GetCodeSystemDetailResponseStatus",
    "InternalServerError",
    "ListCodeSystemsResponse",
    "ListCodesResponse",
    "NotFoundError",
    "NotImplementedError",
    "SemanticSearchResponse",
    "SemanticSearchResult",
    "ServiceUnavailableError",
    "TextSearchResponse",
    "TextSearchResult",
    "UnauthorizedError",
    "UploadRequestFormat",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .agent_chat_response import AgentChatResponse
    from .agent_chat_stream_event import AgentChatStreamEvent
    from .agent_chat_stream_event_type import AgentChatStreamEventType
    from .agent_create_request import AgentCreateRequest
    from .agent_create_request_provider import AgentCreateRequestProvider
    from .agent_delete_response import AgentDeleteResponse
    from .agent_get_chat_messages_request_order import AgentGetChatMessagesRequestOrder
    from .agent_get_chat_messages_request_role import AgentGetChatMessagesRequestRole
    from .agent_get_chat_messages_response import AgentGetChatMessagesResponse
    from .agent_list_response import AgentListResponse
    from .agent_prompts_response import AgentPromptsResponse
    from .agent_response import AgentResponse
    from .agent_template import AgentTemplate
    from .agent_template_provider import AgentTemplateProvider
    from .chat_message_template import ChatMessageTemplate
    from .chat_message_template_role import ChatMessageTemplateRole
    from .chat_session_template import ChatSessionTemplate
    from .json_patch import JsonPatch
    from .json_patch_operation import JsonPatchOperation
    from .json_patch_operation_op import JsonPatchOperationOp
    from .prompt_template import PromptTemplate
    from .success_response import SuccessResponse
_dynamic_imports: typing.Dict[str, str] = {
    "AgentChatResponse": ".agent_chat_response",
    "AgentChatStreamEvent": ".agent_chat_stream_event",
    "AgentChatStreamEventType": ".agent_chat_stream_event_type",
    "AgentCreateRequest": ".agent_create_request",
    "AgentCreateRequestProvider": ".agent_create_request_provider",
    "AgentDeleteResponse": ".agent_delete_response",
    "AgentGetChatMessagesRequestOrder": ".agent_get_chat_messages_request_order",
    "AgentGetChatMessagesRequestRole": ".agent_get_chat_messages_request_role",
    "AgentGetChatMessagesResponse": ".agent_get_chat_messages_response",
    "AgentListResponse": ".agent_list_response",
    "AgentPromptsResponse": ".agent_prompts_response",
    "AgentResponse": ".agent_response",
    "AgentTemplate": ".agent_template",
    "AgentTemplateProvider": ".agent_template_provider",
    "ChatMessageTemplate": ".chat_message_template",
    "ChatMessageTemplateRole": ".chat_message_template_role",
    "ChatSessionTemplate": ".chat_session_template",
    "JsonPatch": ".json_patch",
    "JsonPatchOperation": ".json_patch_operation",
    "JsonPatchOperationOp": ".json_patch_operation_op",
    "PromptTemplate": ".prompt_template",
    "SuccessResponse": ".success_response",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "AgentChatResponse",
    "AgentChatStreamEvent",
    "AgentChatStreamEventType",
    "AgentCreateRequest",
    "AgentCreateRequestProvider",
    "AgentDeleteResponse",
    "AgentGetChatMessagesRequestOrder",
    "AgentGetChatMessagesRequestRole",
    "AgentGetChatMessagesResponse",
    "AgentListResponse",
    "AgentPromptsResponse",
    "AgentResponse",
    "AgentTemplate",
    "AgentTemplateProvider",
    "ChatMessageTemplate",
    "ChatMessageTemplateRole",
    "ChatSessionTemplate",
    "JsonPatch",
    "JsonPatchOperation",
    "JsonPatchOperationOp",
    "PromptTemplate",
    "SuccessResponse",
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import os
import typing

import httpx
from .core.api_error import ApiError
from .core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .core.logging import LogConfig, Logger
from .core.oauth_token_provider import AsyncOAuthTokenProvider, OAuthTokenProvider
from .environment import PhenomlClientEnvironment

if typing.TYPE_CHECKING:
    from .agent.client import AgentClient, AsyncAgentClient
    from .authtoken.client import AsyncAuthtokenClient, AuthtokenClient
    from .cohort.client import AsyncCohortClient, CohortClient
    from .construe.client import AsyncConstrueClient, ConstrueClient
    from .fhir.client import AsyncFhirClient, FhirClient
    from .fhir_provider.client import AsyncFhirProviderClient, FhirProviderClient
    from .lang2fhir.client import AsyncLang2FhirClient, Lang2FhirClient
    from .summary.client import AsyncSummaryClient, SummaryClient
    from .tools.client import AsyncToolsClient, ToolsClient
    from .workflows.client import AsyncWorkflowsClient, WorkflowsClient


class PhenomlClient:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------

    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    client_id : str
        The client identifier used for authentication.

    client_secret : str
        The client secret used for authentication.

    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.Client]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    # or ...

    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    token : typing.Callable[[], str]
        Authenticate by providing a callable that returns a pre-generated bearer token. In this mode, OAuth client credentials are not required.

    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.Client]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from phenoml import PhenomlClient

    client = PhenomlClient()

    # or ...

    from phenoml import PhenomlClient

    client = PhenomlClient(
        base_url="https://yourhost.com/path/to/api",
        token="YOUR_BEARER_TOKEN",
    )
    """

    @typing.overload
    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: PhenomlClientEnvironment = PhenomlClientEnvironment.DEFAULT,
        instance_url: typing.Optional[str] = None,
        headers: typing.Optional[typing.Dict[str, str]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.Client] = None,
        logging: typing.Optional[typing.Union[LogConfig, Logger]] = None,
        client_id: typing.Optional[str] = os.getenv("PHENOML_CLIENT_ID"),
        client_secret: typing.Optional[str] = os.getenv("PHENOML_CLIENT_SECRET"),
    ): ...
    @typing.overload
    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: PhenomlClientEnvironment = PhenomlClientEnvironment.DEFAULT,
        instance_url: typing.Optional[str] = None,
        headers: typing.Optional[typing.Dict[str, str]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.Client] = None,
        logging: typing.Optional[typing.Union[LogConfig, Logger]] = None,
        token: typing.Callable[[], str],
    ): ...
    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: PhenomlClientEnvironment = PhenomlClientEnvironment.DEFAULT,
        instance_url: typing.Optional[str] = None,
        headers: typing.Optional[typing.Dict[str, str]] = None,
        client_id: typing.Optional[str] = os.getenv("PHENOML_CLIENT_ID"),
        client_secret: typing.Optional[str] = os.getenv("PHENOML_CLIENT_SECRET"),
        token: typing.Optional[typing.Callable[[], str]] = None,
        _token_getter_override: typing.Optional[typing.Callable[[], str]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.Client] = None,
        logging: typing.Optional[typing.Union[LogConfig, Logger]] = None,
    ):
        _defaulted_timeout = (
            timeout if timeout is not None else 60 if httpx_client is None else httpx_client.timeout.read
        )
        if instance_url is not None:
            _instance_url = instance_url if instance_url is not None else "experiment.app.pheno.ml"
            base_url = "https://{instanceUrl}".format(instanceUrl=_instance_url)
        if token is not None:
            self._client_wrapper = SyncClientWrapper(
                base_url=_get_base_url(base_url=base_url, environment=environment),
                headers=headers,
                httpx_client=httpx_client
                if httpx_client is not None
                else httpx.Client(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
                if follow_redirects is not None
                else httpx.Client(timeout=_defaulted_timeout),
                timeout=_defaulted_timeout,
                logging=logging,
                token=_token_getter_override if _token_getter_override is not None else token,
            )
        elif client_id is not None and client_secret is not None:
            oauth_token_provider = OAuthTokenProvider(
                client_id=client_id,
                client_secret=client_secret,
                client_wrapper=SyncClientWrapper(
                    base_url=_get_base_url(base_url=base_url, environment=environment),
                    headers=headers,
                    httpx_client=httpx.Client(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
                    if follow_redirects is not None
                    else httpx.Client(timeout=_defaulted_timeout),
                    timeout=_defaulted_timeout,
                    logging=logging,
                ),
            )
            self._client_wrapper = SyncClientWrapper(
                base_url=_get_base_url(base_url=base_url, environment=environment),
                headers=headers,
                token=_token_getter_override if _token_getter_override is not None else oauth_token_provider.get_token,
                httpx_client=httpx_client
                if httpx_client is not None
                else httpx.Client(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
                if follow_redirects is not None
                else httpx.Client(timeout=_defaulted_timeout),
                timeout=_defaulted_timeout,
                logging=logging,
            )
        else:
            raise ApiError(
                body="The client must be instantiated with either 'token' or both 'client_id' and 'client_secret'"
            )
        self._agent: typing.Optional[AgentClient] = None
        self._authtoken: typing.Optional[AuthtokenClient] = None
        self._cohort: typing.Optional[CohortClient] = None
        self._construe: typing.Optional[ConstrueClient] = None
        self._fhir: typing.Optional[FhirClient] = None
        self._fhir_provider: typing.Optional[FhirProviderClient] = None
        self._lang2fhir: typing.Optional[Lang2FhirClient] = None
        self._summary: typing.Optional[SummaryClient] = None
        self._tools: typing.Optional[ToolsClient] = None
        self._workflows: typing.Optional[WorkflowsClient] = None

    @property
    def agent(self):
        if self._agent is None:
            from .agent.client import AgentClient  # noqa: E402

            self._agent = AgentClient(client_wrapper=self._client_wrapper)
        return self._agent

    @property
    def authtoken(self):
        if self._authtoken is None:
            from .authtoken.client import AuthtokenClient  # noqa: E402

            self._authtoken = AuthtokenClient(client_wrapper=self._client_wrapper)
        return self._authtoken

    @property
    def cohort(self):
        if self._cohort is None:
            from .cohort.client import CohortClient  # noqa: E402

            self._cohort = CohortClient(client_wrapper=self._client_wrapper)
        return self._cohort

    @property
    def construe(self):
        if self._construe is None:
            from .construe.client import ConstrueClient  # noqa: E402

            self._construe = ConstrueClient(client_wrapper=self._client_wrapper)
        return self._construe

    @property
    def fhir(self):
        if self._fhir is None:
            from .fhir.client import FhirClient  # noqa: E402

            self._fhir = FhirClient(client_wrapper=self._client_wrapper)
        return self._fhir

    @property
    def fhir_provider(self):
        if self._fhir_provider is None:
            from .fhir_provider.client import FhirProviderClient  # noqa: E402

            self._fhir_provider = FhirProviderClient(client_wrapper=self._client_wrapper)
        return self._fhir_provider

    @property
    def lang2fhir(self):
        if self._lang2fhir is None:
            from .lang2fhir.client import Lang2FhirClient  # noqa: E402

            self._lang2fhir = Lang2FhirClient(client_wrapper=self._client_wrapper)
        return self._lang2fhir

    @property
    def summary(self):
        if self._summary is None:
            from .summary.client import SummaryClient  # noqa: E402

            self._summary = SummaryClient(client_wrapper=self._client_wrapper)
        return self._summary

    @property
    def tools(self):
        if self._tools is None:
            from .tools.client import ToolsClient  # noqa: E402

            self._tools = ToolsClient(client_wrapper=self._client_wrapper)
        return self._tools

    @property
    def workflows(self):
        if self._workflows is None:
            from .workflows.client import WorkflowsClient  # noqa: E402

            self._workflows = WorkflowsClient(client_wrapper=self._client_wrapper)
        return self._workflows


class AsyncPhenomlClient:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------

    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    client_id : str
        The client identifier used for authentication.

    client_secret : str
        The client secret used for authentication.

    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.AsyncClient]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    # or ...

    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    token : typing.Callable[[], str]
        Authenticate by providing a callable that returns a pre-generated bearer token. In this mode, OAuth client credentials are not required.

    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.AsyncClient]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from phenoml import AsyncPhenomlClient

    client = AsyncPhenomlClient()

    # or ...

    from phenoml import AsyncPhenomlClient

    client = AsyncPhenomlClient(
        base_url="https://yourhost.com/path/to/api",
        token="YOUR_BEARER_TOKEN",
    )
    """

    @typing.overload
    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: PhenomlClientEnvironment = PhenomlClientEnvironment.DEFAULT,
        instance_url: typing.Optional[str] = None,
        headers: typing.Optional[typing.Dict[str, str]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.AsyncClient] = None,
        logging: typing.Optional[typing.Union[LogConfig, Logger]] = None,
        client_id: typing.Optional[str] = os.getenv("PHENOML_CLIENT_ID"),
        client_secret: typing.Optional[str] = os.getenv("PHENOML_CLIENT_SECRET"),
    ): ...
    @typing.overload
    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: PhenomlClientEnvironment = PhenomlClientEnvironment.DEFAULT,
        instance_url: typing.Optional[str] = None,
        headers: typing.Optional[typing.Dict[str, str]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.AsyncClient] = None,
        logging: typing.Optional[typing.Union[LogConfig, Logger]] = None,
        token: typing.Callable[[], str],
    ): ...
    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: PhenomlClientEnvironment = PhenomlClientEnvironment.DEFAULT,
        instance_url: typing.Optional[str] = None,
        headers: typing.Optional[typing.Dict[str, str]] = None,
        client_id: typing.Optional[str] = os.getenv("PHENOML_CLIENT_ID"),
        client_secret: typing.Optional[str] = os.getenv("PHENOML_CLIENT_SECRET"),
        token: typing.Optional[typing.Callable[[], str]] = None,
        _token_getter_override: typing.Optional[typing.Callable[[], str]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.AsyncClient] = None,
        logging: typing.Optional[typing.Union[LogConfig, Logger]] = None,
    ):
        _defaulted_timeout = (
            timeout if timeout is not None else 60 if httpx_client is None else httpx_client.timeout.read
        )
        if instance_url is not None:
            _instance_url = instance_url if instance_url is not None else "experiment.app.pheno.ml"
            base_url = "https://{instanceUrl}".format(instanceUrl=_instance_url)
        if token is not None:
            self._client_wrapper = AsyncClientWrapper(
                base_url=_get_base_url(base_url=base_url, environment=environment),
                headers=headers,
                httpx_client=httpx_client
                if httpx_client is not None
                else httpx.AsyncClient(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
                if follow_redirects is not None
                else httpx.AsyncClient(timeout=_defaulted_timeout),
                timeout=_defaulted_timeout,
                logging=logging,
                token=_token_getter_override if _token_getter_override is not None else token,
            )
        elif client_id is not None and client_secret is not None:
            oauth_token_provider = AsyncOAuthTokenProvider(
                client_id=client_id,
                client_secret=client_secret,
                client_wrapper=AsyncClientWrapper(
                    base_url=_get_base_url(base_url=base_url, environment=environment),
                    headers=headers,
                    httpx_client=httpx.AsyncClient(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
                    if follow_redirects is not None
                    else httpx.AsyncClient(timeout=_defaulted_timeout),
                    timeout=_defaulted_timeout,
                    logging=logging,
                ),
            )
            self._client_wrapper = AsyncClientWrapper(
                base_url=_get_base_url(base_url=base_url, environment=environment),
                headers=headers,
                token=_token_getter_override,
                async_token=oauth_token_provider.get_token,
                httpx_client=httpx_client
                if httpx_client is not None
                else httpx.AsyncClient(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
                if follow_redirects is not None
                else httpx.AsyncClient(timeout=_defaulted_timeout),
                timeout=_defaulted_timeout,
                logging=logging,
            )
        else:
            raise ApiError(
                body="The client must be instantiated with either 'token' or both 'client_id' and 'client_secret'"
            )
        self._agent: typing.Optional[AsyncAgentClient] = None
        self._authtoken: typing.Optional[AsyncAuthtokenClient] = None
        self._cohort: typing.Optional[AsyncCohortClient] = None
        self._construe: typing.Optional[AsyncConstrueClient] = None
        self._fhir: typing.Optional[AsyncFhirClient] = None
        self._fhir_provider: typing.Optional[AsyncFhirProviderClient] = None
        self._lang2fhir: typing.Optional[AsyncLang2FhirClient] = None
        self._summary: typing.Optional[AsyncSummaryClient] = None
        self._tools: typing.Optional[AsyncToolsClient] = None
        self._workflows: typing.Optional[AsyncWorkflowsClient] = None

    @property
    def agent(self):
        if self._agent is None:
            from .agent.client import AsyncAgentClient  # noqa: E402

            self._agent = AsyncAgentClient(client_wrapper=self._client_wrapper)
        return self._agent

    @property
    def authtoken(self):
        if self._authtoken is None:
            from .authtoken.client import AsyncAuthtokenClient  # noqa: E402

            self._authtoken = AsyncAuthtokenClient(client_wrapper=self._client_wrapper)
        return self._authtoken

    @property
    def cohort(self):
        if self._cohort is None:
            from .cohort.client import AsyncCohortClient  # noqa: E402

            self._cohort = AsyncCohortClient(client_wrapper=self._client_wrapper)
        return self._cohort

    @property
    def construe(self):
        if self._construe is None:
            from .construe.client import AsyncConstrueClient  # noqa: E402

            self._construe = AsyncConstrueClient(client_wrapper=self._client_wrapper)
        return self._construe

    @property
    def fhir(self):
        if self._fhir is None:
            from .fhir.client import AsyncFhirClient  # noqa: E402

            self._fhir = AsyncFhirClient(client_wrapper=self._client_wrapper)
        return self._fhir

    @property
    def fhir_provider(self):
        if self._fhir_provider is None:
            from .fhir_provider.client import AsyncFhirProviderClient  # noqa: E402

            self._fhir_provider = AsyncFhirProviderClient(client_wrapper=self._client_wrapper)
        return self._fhir_provider

    @property
    def lang2fhir(self):
        if self._lang2fhir is None:
            from .lang2fhir.client import AsyncLang2FhirClient  # noqa: E402

            self._lang2fhir = AsyncLang2FhirClient(client_wrapper=self._client_wrapper)
        return self._lang2fhir

    @property
    def summary(self):
        if self._summary is None:
            from .summary.client import AsyncSummaryClient  # noqa: E402

            self._summary = AsyncSummaryClient(client_wrapper=self._client_wrapper)
        return self._summary

    @property
    def tools(self):
        if self._tools is None:
            from .tools.client import AsyncToolsClient  # noqa: E402

            self._tools = AsyncToolsClient(client_wrapper=self._client_wrapper)
        return self._tools

    @property
    def workflows(self):
        if self._workflows is None:
            from .workflows.client import AsyncWorkflowsClient  # noqa: E402

            self._workflows = AsyncWorkflowsClient(client_wrapper=self._client_wrapper)
        return self._workflows


def _get_base_url(*, base_url: typing.Optional[str] = None, environment: PhenomlClientEnvironment) -> str:
    if base_url is not None:
        return base_url
    elif environment is not None:
        return environment.value
    else:
        raise Exception("Please pass in either base_url or environment to construct the client")
